==> junitify-0.1.3.crate: junitify-0.1.3/src/main.rs <==
//     junitify - Takes cargo test JSON and transform to JUnit XML
//
//         The MIT License (MIT)
//
//      Copyright (c) KoresFramework (https://gitlab.com/Kores/)
//      Copyright (c) contributors
//
//      Permission is hereby granted, free of charge, to any person obtaining a copy
//      of this software and associated documentation files (the "Software"), to deal
//      in the Software without restriction, including without limitation the rights
//      to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//      copies of the Software, and to permit persons to whom the Software is
//      furnished to do so, subject to the following conditions:
//
//      The above copyright notice and this permission notice shall be included in
//      all copies or substantial portions of the Software.
//
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//      OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//      THE SOFTWARE.
mod test_parser;
mod test_producer;

use crate::test_parser::{ParsedTestSuite, TestParser};
use crate::test_producer::{RenderedSuite, TestProducer};
use clap::{AppSettings, Clap};
use handlebars::RenderError;
use serde_json::Error;
use std::fmt::Debug;
use std::fs::OpenOptions;
use std::io::{self, Write};
use std::path::PathBuf;

#[derive(Clap)]
#[clap(version = "1.0", author = "Jonathan H. R. Lopes <jhrldev@gmail.com>")]
#[clap(setting = AppSettings::ColoredHelp)]
struct Opts {
    #[clap(short, long, about = "Output directory for produced .xml files.")]
    out: Option<String>,
    #[clap(short, long, about = "Mirror stdin in stdout.")]
    mirror: bool,
}

#[derive(Debug)]
enum AppError {
    Serde(Error),
    Template(RenderError),
    IO(std::io::Error),
}

fn main() -> Result<(), AppError> {
    let opts: Opts = Opts::parse();
    let render = render_from_stdin(opts.mirror)?;

    if let Some(ref out) = opts.out {
        let root = PathBuf::from(out);
        if !root.exists() {
            std::fs::create_dir(root.clone()).map_err(|e| AppError::IO(e))?;
        }

        for i in 0..render.len() {
            let rendered = render.get(i).unwrap();
            let path = root.join(format!("{}-{}.xml", rendered.name, i));

            let mut file = OpenOptions::new()
                .create(true)
                .write(true)
                .truncate(true)
                .open(path)
                .map_err(|e| AppError::IO(e))?;

            file.write_all(rendered.rendered.as_bytes())
                .map_err(|e| AppError::IO(e))?;
        }
    } else {
        for r in render {
            println!("<{}>", r.name);
            println!("{}", r.rendered);
            println!("</{}>", r.name);
        }
    }
    Ok(())
}

fn render_from_stdin(mirror: bool) -> Result<Vec<RenderedSuite>, AppError> {
    let mut buffer = String::new();
    let stdin = io::stdin();

    let mut suites: Vec<ParsedTestSuite> = vec![];
    let mut parser = TestParser::new();
    let mut line;
    loop {
        line = stdin.read_line(&mut buffer);
        if let Ok(l) = line {
            if l > 0 && buffer.len() > 0 {
                if mirror {
                    println!("{}", buffer)
                }

                let parse = parser.parse(&buffer);
                if let Ok(p) = parse {
                    if let Some(suite) = p {
                        suites.push(suite);
                        parser.reset();
                    }
                    buffer.clear();
                } else if let Err(e) = parse {
                    return Err(AppError::Serde(e));
                }
            } else {
                break;
            }
        } else {
            break;
        }
    }

    TestProducer::produce(suites).map_err(|e| AppError::Template(e))
}

#[cfg(test)]
mod tests {
    use crate::test_parser::parse_test;
    use crate::test_parser::Test::Test;
    use crate::{parse_test, Test};

    #[test]
    fn suite() {
        let suite = r#"{ "type": "suite", "event": "started", "test_count": 14 }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::new_suite_started("started".to_string(), 14)),
            result
        );
    }

    #[test]
    fn test() {
        let suite =
            r#"{ "type": "test", "event": "started", "name": "tests::bench_1mb_tar_detect" }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::Test {
                event: "started".to_string(),
                name: "tests::bench_1mb_tar_detect".to_string()
            }),
            result
        );
    }

    #[test]
    fn suite_ok() {
        let suite = r#"{ "type": "suite", "event": "ok", "passed": 1, "failed": 0, "allowed_fail": 0, "ignored": 0, "measured": 0, "filtered_out": 0, "exec_time": 0.279672772 }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::new_suite_ok(
                "ok".to_string(),
                1,
                0,
                0,
                0,
                0,
                0,
                0.279672772f64
            )),
            result
        );
    }
}

==> junitify-0.1.6.crate: junitify-0.1.6/src/main.rs <==
//     junitify - Takes cargo test JSON and transform to JUnit XML
//
//         The MIT License (MIT)
//
//      Copyright (c) KoresFramework (https://gitlab.com/Kores/)
//      Copyright (c) contributors
//
//      Permission is hereby granted, free of charge, to any person obtaining a copy
//      of this software and associated documentation files (the "Software"), to deal
//      in the Software without restriction, including without limitation the rights
//      to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//      copies of the Software, and to permit persons to whom the Software is
//      furnished to do so, subject to the following conditions:
//
//      The above copyright notice and this permission notice shall be included in
//      all copies or substantial portions of the Software.
//
//      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//      IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//      FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//      AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//      LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//      OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//      THE SOFTWARE.
mod test_parser;
mod test_producer;

use crate::test_parser::{ParsedTestSuite, TestParser};
use crate::test_producer::{RenderedSuite, TestProducer};
use clap::{AppSettings, Clap};
use handlebars::RenderError;
use serde_json::Error;
use std::fmt::Debug;
use std::fs::OpenOptions;
use std::io::{self, Write};
use std::path::PathBuf;

#[derive(Clap)]
#[clap(version = "1.0", author = "Jonathan H. R. Lopes <jhrldev@gmail.com>")]
#[clap(setting = AppSettings::ColoredHelp)]
struct Opts {
    #[clap(short, long, about = "Output directory for produced .xml files.")]
    out: Option<String>,
    #[clap(short, long, about = "Mirror stdin in stdout.")]
    mirror: bool,
}

#[derive(Debug)]
enum AppError {
    Serde(Error),
    Template(RenderError),
    IO(std::io::Error),
}

fn main() -> Result<(), AppError> {
    let opts: Opts = Opts::parse();
    let render = render_from_stdin(opts.mirror)?;

    if let Some(ref out) = opts.out {
        let root = PathBuf::from(out);
        if !root.exists() {
            std::fs::create_dir(root.clone()).map_err(|e| AppError::IO(e))?;
        }

        for i in 0..render.len() {
            let rendered = render.get(i).unwrap();
            let path = root.join(format!("{}-{}.xml", rendered.name, i));

            let mut file = OpenOptions::new()
                .create(true)
                .write(true)
                .truncate(true)
                .open(path)
                .map_err(|e| AppError::IO(e))?;

            file.write_all(rendered.rendered.as_bytes())
                .map_err(|e| AppError::IO(e))?;
        }
    } else {
        for r in render {
            println!("<{}>", r.name);
            println!("{}", r.rendered);
            println!("</{}>", r.name);
        }
    }
    Ok(())
}

fn render_from_stdin(mirror: bool) -> Result<Vec<RenderedSuite>, AppError> {
    let mut buffer = String::new();
    let stdin = io::stdin();

    let mut suites: Vec<ParsedTestSuite> = vec![];
    let mut parser = TestParser::new();
    let mut line;
    loop {
        line = stdin.read_line(&mut buffer);
        if let Ok(l) = line {
            if l > 0 && buffer.len() > 0 {
                if mirror {
                    println!("{}", buffer)
                }

                let parse = parser.parse(&buffer);
                if let Ok(p) = parse {
                    if let Some(suite) = p {
                        suites.push(suite);
                        parser.reset();
                    }
                    buffer.clear();
                } else if let Err(e) = parse {
                    return Err(AppError::Serde(e));
                }
            } else {
                break;
            }
        } else {
            break;
        }
    }

    TestProducer::produce(suites).map_err(|e| AppError::Template(e))
}

#[cfg(test)]
mod tests {
    use crate::test_parser::{parse_test, Test};

    #[test]
    fn suite() {
        let suite = r#"{ "type": "suite", "event": "started", "test_count": 14 }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::new_suite_started("started".to_string(), 14)),
            result
        );
    }

    #[test]
    fn test() {
        let suite =
            r#"{ "type": "test", "event": "started", "name": "tests::bench_1mb_tar_detect" }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::Test {
                event: "started".to_string(),
                name: "tests::bench_1mb_tar_detect".to_string(),
                stdout: None
            }),
            result
        );
    }

    #[test]
    fn suite_ok() {
        let suite = r#"{ "type": "suite", "event": "ok", "passed": 1, "failed": 0, "allowed_fail": 0, "ignored": 0, "measured": 0, "filtered_out": 0, "exec_time": 0.279672772 }"#;
        let result = parse_test(suite).map_err(|e| e.to_string());
        assert_eq!(
            Ok(Test::new_suite_ok(
                "ok".to_string(),
                1,
                0,
                0,
                0,
                0,
                0,
                0.279672772f64
            )),
            result
        );
    }
}
